(*----------------------------------------------------------------------*)

BeginPackage["HTessellate`", {"HDraw`", "HTriangles`", 
                              "HPolygons`", "Global`"}]

(* 
  
  This package defines some routines for calculating and visualizing
  Euclidean and hyperbolic geometry in the unit disk.
  
  This package was developed for Mathematica version 2.2.
  
  The newest version of this package is available for anonymous ftp at
  nic.funet.fi, directory "pub/math/riemann/mathematica".
  
  Version 0.1. Last modified: Nov 1st, 1994.
  
  Send comments and bug reports to 
  
    Juha Haataja (e-mail: jhaataja@csc.fi)
    Center for Scientific Computing
    PO Box 405
    FIN-02101 Espoo, Finland
  
*)

(*----------------------------------------------------------------------*)

HTessellate::usage = 
"HTessellate is a collection of packages for forming, manipulating and 
visualizing geometric objects in two-dimensional hyperbolic 
geometry. See the documentation for the command HDraw.";

(*----------------------------------------------------------------------*)

Message[HTessellate::usage]

(*----------------------------------------------------------------------*)

EndPackage[];

(*----------------------------------------------------------------------*)


